@@ -5,11 +5,12 @@
 #' @include DSM.Analysis.R
 #' @include generic.functions.R
 #' @include LT.Survey.Results.R
+#' @include Survey.Results.R
 #' @include DDF.Data.R
 
-#' Class "Simulation"
+#' @title Class "Simulation"
 #'
-#' Class \code{"Simulation"} is an S4 class containing descriptions of the
+#' @description Class \code{"Simulation"} is an S4 class containing descriptions of the
 #' region, population, survey design and analyses the user wishes to investigate.
 #' Once the simulation has been run the N.D.Estimates will contain multiple
 #' estimates of abundance and density obtained by repeatedly generating
@@ -102,25 +103,27 @@ setMethod(
 )
 
 setValidity("Simulation",
-  function(object){
-    if(object@double.observer){
-      warning("Double observer simulations not supported at present", call. = TRUE, immediate. = TRUE)
-      return(FALSE)
-    }
-    design <- object@design
-    transects.from.file <- ifelse(length(design@path) == 1, TRUE, FALSE)
-    if(transects.from.file & !object@single.transect.set){
-      no.files <- length(design@filenames)
-      if(object@reps > no.files){
-        message("You have specified a higher number of repetitions than you have provided transect shapefiles at: ", design@path)
-        return(FALSE)
-      }
-    }else if(!transects.from.file){
-      message("The generation of transects is not currently implemented in R")
-      return(FALSE)
-    }
-    return(TRUE)
-  }
+            function(object){
+              if(object@double.observer){
+                warning("Double observer simulations not supported at present", call. = TRUE, immediate. = TRUE)
+                return(FALSE)
+              }
+              design <- object@design
+              transects.from.file <- ifelse(length(design@path) == 1, TRUE, FALSE)
+              if(transects.from.file & !object@single.transect.set){
+                no.files <- length(design@filenames)
+                if(object@reps > no.files){
+                  message("You have specified a higher number of repetitions than you have provided transect shapefiles at: ", design@path)
+                  return(FALSE)
+                }
+              }else if(!transects.from.file){
+                if(!(class(object@design) %in% c("PT.Nested.Design"))){
+                  message("The generation of transects is only implemented in R for nested point transect designs")
+                  return(FALSE)  
+                }
+              }
+              return(TRUE)
+            }
 )
 
 # GENERIC METHODS DEFINITIONS --------------------------------------------
@@ -143,7 +146,14 @@ setMethod(
     if(description.summary){
       description.summary()
     }
-    reps <- dim(object@results$individuals$N)[3]-2
+    #Create function to calculate RMSE
+    calc.RMSE <- function(x, reps){ 
+      true.x <- x[(reps+1)]
+      x <- x[1:reps]
+      return( sqrt( sum((x-true.x)^2) / reps ))
+    }
+    #Get number of reps
+    reps <- object@reps
     #Calculate true values
     strata.names <- object@region@strata.name
     strata.order <- NULL
@@ -166,7 +176,7 @@ setMethod(
       #Add on totals
       areas <- c(areas, sum(areas))
       N <- c(N, sum(N))
-    #Otherwise process areas and Population size as normal
+      #Otherwise process areas and Population size as normal
     }else{
       #Re ordering in the same way as the results tables (think dht arranges them)
       for(strat in seq(along = strata.names)){
@@ -198,7 +208,7 @@ setMethod(
       true.D.individuals <- true.N.individuals/areas
       true.D.clusters <- true.N.clusters/areas
     }
-
+    
     #Create summary tables
     capture <- array(NA, dim = c(reps, length(true.N.individuals)))
     capture.D <- array(NA, dim = c(reps, length(true.D.individuals)))
@@ -214,6 +224,13 @@ setMethod(
     percent.capture <- (apply(capture, 2, sum, na.rm = TRUE)/nrow(na.omit(capture)))*100
     percent.capture.D <- (apply(capture.D, 2, sum, na.rm = TRUE)/nrow(na.omit(capture)))*100
     zero.n <- apply(zero.n, 2, sum)
+    if(length(true.N.individuals) == 1){
+      RMSE.N = apply(cbind(t(as.matrix(object@results$individuals$N[, "Estimate", 1:reps])), true.N.individuals), 1, calc.RMSE, reps = reps)
+      RMSE.D = apply(cbind(t(as.matrix(object@results$individuals$D[, "Estimate", 1:reps])), true.D.individuals), 1, calc.RMSE, reps = reps)
+    }else{
+      RMSE.N = apply(cbind(object@results$individuals$N[, "Estimate", 1:reps], true.N.individuals), 1, calc.RMSE, reps = reps) 
+      RMSE.D = apply(cbind(object@results$individuals$D[, "Estimate", 1:reps], true.D.individuals), 1, calc.RMSE, reps = reps)
+    }
     individual.summary <- data.frame(mean.Cover.Area = object@results$individuals$summary[,"CoveredArea","mean"],
                                      mean.Effort = object@results$individuals$summary[,"Effort","mean"],
                                      mean.n = object@results$individuals$summary[,"n","mean"],
@@ -222,26 +239,28 @@ setMethod(
                                      mean.se.ER = object@results$individuals$summary[,"se.ER","mean"],
                                      sd.mean.ER = object@results$individuals$summary[,"ER","sd"])
     individual.N <- data.frame(Truth = true.N.individuals,
-                                   mean.Estimate = object@results$individuals$N[,"Estimate","mean"],
-                                   percent.bias = (object@results$individuals$N[,"Estimate","mean"] - true.N.individuals)/true.N.individuals*100,
-                                   #lcl = object@results$individuals$N[,"lcl","mean"],
-                                   #ucl = object@results$individuals$N[,"ucl","mean"],
-                                   CI.coverage.prob = percent.capture/100,
-                                   mean.se = object@results$individuals$N[,"se","mean"],
-                                   sd.of.means = object@results$individuals$N[,"Estimate","sd"])
+                               mean.Estimate = object@results$individuals$N[,"Estimate","mean"],
+                               percent.bias = (object@results$individuals$N[,"Estimate","mean"] - true.N.individuals)/true.N.individuals*100,
+                               RMSE = RMSE.N,
+                               #lcl = object@results$individuals$N[,"lcl","mean"],
+                               #ucl = object@results$individuals$N[,"ucl","mean"],
+                               CI.coverage.prob = percent.capture/100,
+                               mean.se = object@results$individuals$N[,"se","mean"],
+                               sd.of.means = object@results$individuals$N[,"Estimate","sd"])
     individual.D <- data.frame(Truth = true.D.individuals,
-                                   mean.Estimate = object@results$individuals$D[,"Estimate","mean"],
-                                   percent.bias = (object@results$individuals$D[,"Estimate","mean"] - true.D.individuals)/true.D.individuals*100,
-                                   #lcl = object@results$individuals$N[,"lcl","mean"],
-                                   #ucl = object@results$individuals$N[,"ucl","mean"],
-                                   CI.coverage.prob = percent.capture.D/100,
-                                   mean.se = object@results$individuals$D[,"se","mean"],
-                                   sd.of.means = object@results$individuals$D[,"Estimate","sd"])
-
+                               mean.Estimate = object@results$individuals$D[,"Estimate","mean"],
+                               percent.bias = (object@results$individuals$D[,"Estimate","mean"] - true.D.individuals)/true.D.individuals*100,
+                               RMSE = RMSE.D,
+                               #lcl = object@results$individuals$N[,"lcl","mean"],
+                               #ucl = object@results$individuals$N[,"ucl","mean"],
+                               CI.coverage.prob = percent.capture.D/100,
+                               mean.se = object@results$individuals$D[,"se","mean"],
+                               sd.of.means = object@results$individuals$D[,"Estimate","sd"])
+    
     if(!is.null(object@results$clusters)){
-      capture <- array(NA, dim = c(reps, length(true.N.individuals)))
-      capture.D <- array(NA, dim = c(reps, length(true.D.individuals)))
-      zero.n <- array(NA, dim = c(reps, length(true.N.individuals)))
+      capture <- array(NA, dim = c(reps, length(true.N.clusters)))
+      capture.D <- array(NA, dim = c(reps, length(true.D.clusters)))
+      zero.n <- array(NA, dim = c(reps, length(true.N.clusters)))
       for(strat in seq(along = true.N.clusters)){
         for(i in 1:reps){
           capture[i, strat] <- ifelse(object@results$clusters$N[strat, "lcl", i] < true.N.clusters[strat] & object@results$clusters$N[strat, "ucl", i] > true.N.clusters[strat], TRUE, FALSE)
@@ -252,30 +271,39 @@ setMethod(
       percent.capture <- (apply(capture, 2, sum, na.rn = TRUE)/nrow(na.omit(capture)))*100
       percent.capture.D <- (apply(capture.D, 2, sum, na.rm = TRUE)/nrow(na.omit(capture.D)))*100
       zero.n <- apply(zero.n, 2, sum)
+      if(length(true.N.clusters) == 1){
+        RMSE.N = apply(cbind(t(as.matrix(object@results$clusters$N[, "Estimate", 1:reps])), true.N.clusters), 1, calc.RMSE, reps = reps)
+        RMSE.D = apply(cbind(t(as.matrix(object@results$clusters$D[, "Estimate", 1:reps])), true.D.clusters), 1, calc.RMSE, reps = reps)
+      }else{
+        RMSE.N = apply(cbind(object@results$clusters$N[, "Estimate", 1:reps], true.N.clusters), 1, calc.RMSE, reps = reps) 
+        RMSE.D = apply(cbind(object@results$clusters$D[, "Estimate", 1:reps], true.D.clusters), 1, calc.RMSE, reps = reps)
+      }
       cluster.summary <- data.frame(mean.Cover.Area = object@results$clusters$summary[,"CoveredArea","mean"],
-                                       mean.Effort = object@results$clusters$summary[,"Effort","mean"],
-                                       mean.n = object@results$clusters$summary[,"n","mean"],
-                                       no.zero.n = zero.n,
-                                       mean.k = object@results$clusters$summary[,"k","mean"],
-                                       mean.ER = object@results$clusters$summary[,"ER","mean"],
-                                       mean.se.ER = object@results$clusters$summary[,"se.ER","mean"],
-                                       sd.mean.ER = object@results$clusters$summary[,"ER","sd"])
+                                    mean.Effort = object@results$clusters$summary[,"Effort","mean"],
+                                    mean.n = object@results$clusters$summary[,"n","mean"],
+                                    no.zero.n = zero.n,
+                                    mean.k = object@results$clusters$summary[,"k","mean"],
+                                    mean.ER = object@results$clusters$summary[,"ER","mean"],
+                                    mean.se.ER = object@results$clusters$summary[,"se.ER","mean"],
+                                    sd.mean.ER = object@results$clusters$summary[,"ER","sd"])
       cluster.N <- data.frame(Truth = true.N.clusters,
-                                     mean.Estimate = object@results$clusters$N[,"Estimate","mean"],
-                                     percent.bias = (object@results$clusters$N[,"Estimate","mean"] - true.N.clusters)/true.N.clusters*100,
-                                     #lcl = object@results$clusters$N[,"lcl","mean"],
-                                     #ucl = object@results$clusters$N[,"ucl","mean"],
-                                     CI.coverage.prob = percent.capture/100,
-                                     mean.se = object@results$clusters$N[,"se","mean"],
-                                     sd.of.means = object@results$clusters$N[,"Estimate","sd"])
+                              mean.Estimate = object@results$clusters$N[,"Estimate","mean"],
+                              percent.bias = (object@results$clusters$N[,"Estimate","mean"] - true.N.clusters)/true.N.clusters*100,
+                              RMSE = RMSE.N,
+                              #lcl = object@results$clusters$N[,"lcl","mean"],
+                              #ucl = object@results$clusters$N[,"ucl","mean"],
+                              CI.coverage.prob = percent.capture/100,
+                              mean.se = object@results$clusters$N[,"se","mean"],
+                              sd.of.means = object@results$clusters$N[,"Estimate","sd"])
       cluster.D <- data.frame(Truth = true.D.clusters,
-                                     mean.Estimate = object@results$clusters$D[,"Estimate","mean"],
-                                     percent.bias = abs(object@results$clusters$D[,"Estimate","mean"] - true.D.clusters)/true.D.clusters*100,
-                                     #lcl = object@results$clusters$N[,"lcl","mean"],
-                                     #ucl = object@results$clusters$N[,"ucl","mean"],
-                                     CI.coverage.prob = percent.capture.D/100,
-                                     mean.se = object@results$clusters$D[,"se","mean"],
-                                     sd.of.means = object@results$clusters$D[,"Estimate","sd"])
+                              mean.Estimate = object@results$clusters$D[,"Estimate","mean"],
+                              percent.bias = abs(object@results$clusters$D[,"Estimate","mean"] - true.D.clusters)/true.D.clusters*100,
+                              RMSE = RMSE.D,
+                              #lcl = object@results$clusters$N[,"lcl","mean"],
+                              #ucl = object@results$clusters$N[,"ucl","mean"],
+                              CI.coverage.prob = percent.capture.D/100,
+                              mean.se = object@results$clusters$D[,"se","mean"],
+                              sd.of.means = object@results$clusters$D[,"Estimate","sd"])
       expected.size <- data.frame(Truth = true.expected.s,
                                   mean.Expected.S = object@results$expected.size[,"Expected.S","mean"],
                                   percent.bias = abs(true.expected.s - object@results$expected.size[,"Expected.S","mean"])/true.expected.s*100,
@@ -292,10 +320,45 @@ setMethod(
     no.fails <- length(which(is.na(object@results$Detection[1,1,1:object@reps])))
     #print(individual.N.est)
     individuals <- list(summary = individual.summary, N = individual.N, D = individual.D)
+    #Model selection table
+    tab.model.selection <- table(object@results$Detection[,"SelectedModel",1:object@reps])
+    #Create detectabilty summary
+    detectability.summary <- list(key.function = object@detectability@key.function, scale.param = object@detectability@scale.param, shape.param = object@detectability@shape.param, truncation = object@detectability@truncation)
+    #Create analysis summary
+    analysis.summary <- list(dsmodels = list(), criteria = object@ddf.analyses[[1]]@criteria, truncation = object@ddf.analyses[[1]]@truncation)
+    #Create design summary
+    design.type <- switch(class(object@design),
+                          LT.Systematic.Design = "Systematic Parallel Line Transect",
+                          LT.EqAngle.ZZ.Design = "Equal Angle Zigzag Line Transect",
+                          LT.EqSpace.ZZ.Design = "Equal Spaced Zigzag Line Transect",
+                          LT.Random.Design = "Random Parallel Line Transect",
+                          LT.User.Specified.Design = "Subjective Line Transect",
+                          PT.Systematic.Design = "Systematic Point Transect",
+                          PT.Nested.Design = "Systematic Nested Point Transect",
+                          PT.Random.Design = "Random Point Transect")
+    slots <- slotNames(object@design)
+    design.parameters <- list()
+    for(i in seq(along = slots)){
+      if(slots[i] %in% c("design.axis", "spacing", "plus.sampling", "nested.space")){
+        design.parameters[[slots[i]]] <- slot(object@design, slots[i])  
+      }
+    }
+    design.summary <- new(Class = "Design.Summary", design.type = design.type, design.parameters = design.parameters)
+    #Create population summary
+    
+    if(object@ddf.analyses[[1]]@binned.data){
+      analysis.summary$cutpoints <- object@ddf.analyses[[1]]@cutpoints
+    }
+    if(nrow(analysis.strata) > 0){
+      analysis.summary$analysis.strata <- object@ddf.analyses[[1]]@analysis.strata
+    }
+    for(i in seq(along = object@ddf.analyses)){
+      analysis.summary$dsmodels[[i]] <- object@ddf.analyses[[i]]@dsmodel
+    }
     if(!is.null(object@results$clusters)){
-      summary.x <- new(Class = "Simulation.Summary", region.name = object@region@region.name, total.reps = object@reps, failures = no.fails, individuals = individuals, clusters = clusters, expected.size = expected.size, detection = detection)
+      summary.x <- new(Class = "Simulation.Summary", region.name = object@region@region.name, total.reps = object@reps, failures = no.fails, individuals = individuals, clusters = clusters, expected.size = expected.size, detection = detection, model.selection = tab.model.selection, design.summary = design.summary, detectability.summary = detectability.summary, analysis.summary = analysis.summary)
     }else{
-      summary.x <- new(Class = "Simulation.Summary", region.name = object@region@region.name, total.reps = object@reps, failures = no.fails, individuals = individuals, detection = detection)
+      summary.x <- new(Class = "Simulation.Summary", region.name = object@region@region.name, total.reps = object@reps, failures = no.fails, individuals = individuals, detection = detection, model.selection = tab.model.selection, design.summary = design.summary, detectability.summary = detectability.summary, analysis.summary = analysis.summary)
     }
     return(summary.x)
   }
@@ -317,6 +380,29 @@ setMethod(
   }
 )
 
+#' histogram.N.ests
+#' 
+#' Plots a histogram of the estimates abundances
+#' 
+#' @param x object of class Simulation
+#' @param ... optional parameters to pass to the generic hist function in graphics
+#' @rdname histogram.N.ests-methods
+#' @importFrom graphics hist abline
+#' @export
+histogram.N.ests <- function(x, ...){
+  reps <- x@reps
+  index <- dim(x@results$individuals$N)[1]
+  true.N <- sum(x@population.description@N)
+  if(!is.null(x@results$clusters)){
+    ests <- x@results$clusters$N[index, "Estimate", 1:reps]
+    hist(ests, main = "Histogram of Estimates", xlab = "Estimated Abundance of Clusters", ...) 
+  }else{
+    ests <- x@results$individuals$N[index, "Estimate", 1:reps]
+    hist(ests, main = "Histogram of Estimates", xlab = "Estimated Abundance of Individuals", ...)  
+  }
+  abline(v = true.N, col = 2, lwd = 3, lty = 2)
+  invisible(x)
+}
 
 # @rdname Simulation-class
 # @export
@@ -366,6 +452,8 @@ setMethod(
     }else{
       if(inherits(object@design, "LT.Design")){
         survey <- new(Class = "Single.Obs.LT.Survey", population = population, line.transect = transects, perp.truncation = object@detectability@truncation)
+      }else if(inherits(object@design, "PT.Design")){
+        survey <- new(Class = "Single.Obs.PT.Survey", population = population, point.transect = transects, rad.truncation = object@detectability@truncation)
       }
     }
     survey.data <- create.survey.results(object = survey, dht.tables = dht.tables, region = object@region)
@@ -374,12 +462,12 @@ setMethod(
       obs.table <- survey.data$obs.table
       sample.table <- survey.data$sample.table
       region.table <- survey.data$region.table
-      survey.results <- new(Class = "LT.Survey.Results", region = object@region, population = population, transects = transects, ddf.data = ddf.data, obs.table = obs.table, sample.table = sample.table, region.table = region.table)
+      survey.results <- new(Class = "Survey.Results", region = object@region, population = population, transects = transects, ddf.data = ddf.data, obs.table = obs.table, sample.table = sample.table, region.table = region.table)
     }else{
       obs.table <- new(Class = "Obs.Table")
       sample.table <- new(Class = "Sample.Table")
       region.table <- new(Class = "Region.Table")
-      survey.results <- new(Class = "LT.Survey.Results", region = object@region, population = population, transects = transects, ddf.data = ddf.data, obs.table = obs.table, sample.table = sample.table, region.table = region.table)
+      survey.results <- new(Class = "Survey.Results", region = object@region, population = population, transects = transects, ddf.data = ddf.data, obs.table = obs.table, sample.table = sample.table, region.table = region.table)
     }
     return(survey.results)
   }
@@ -390,10 +478,15 @@ setMethod(
 #' @export
 setMethod(
   f="run.analysis",
-  signature=c("Simulation","LT.Survey.Results"),
-  definition=function(object, data, dht = TRUE){
+  signature=c("Simulation","Survey.Results"),
+  definition=function(object, data, dht = FALSE){
     #dist.data <- survey.results@ddf.data
     best.model <- run.analysis(object, data@ddf.data)
+    #If dht is true but tables have not been provided
+    if(dht & nrow(data@region.table@region.table) == 0){
+      warning("dht tables have not been provided please re-run create.survey.results with dht.tables = TRUE if you would like density/abundance estimates in addition to ddf results.", immediate. = TRUE, call. = FALSE)
+      dht = FALSE
+    }
     #If ddf has converged and dht it TRUE
     if(dht & !is.null(best.model)){
       #Calculate density/abundance
@@ -410,22 +503,44 @@ setMethod(
 setMethod(
   f="run.analysis",
   signature=c("Simulation","DDF.Data"),
-  definition=function(object, data, dht = TRUE){
+  definition=function(object, data, dht = FALSE){
     ddf.analyses <- object@ddf.analyses
     criteria <- NULL
     results <- list()
+    point <- inherits(object@design, "PT.Design")
     for(a in seq(along = ddf.analyses)){
-      results[[a]] <- run.analysis(ddf.analyses[[a]], data)
+      results[[a]] <- run.analysis(ddf.analyses[[a]], data, point = point)
       if(!is.na(results[[a]][1])){
-        criteria <- c(criteria, results[[a]]$criterion)
+        #Get information to calculate selection criteria
+        lnl <- results[[a]]$lnl 
+        k <- length(results[[a]]$par)
+        n <- nrow(results[[a]]$data)
+        criterion <- object@ddf.analyses[[1]]@criteria
+        selection.criterion.value <- switch(criterion,
+                                            AIC  = 2*k-2*lnl,
+                                            aic  = 2*k-2*lnl,
+                                            AICc = 2*k-2*lnl+(2*k*(k+1))/(n-k-1),
+                                            BIC  = k*log(n)-2*lnl,
+                                            bic  = k*log(n)-2*lnl)
+        criteria <- c(criteria, selection.criterion.value)
       }else{
         criteria <- c(criteria, NA)
       }
     }
     #check that at least one model worked
-    if(length(which(!is.na(criteria))) > 0){
-      best.model <- which(criteria == min(na.omit(criteria)))
-      return(results[[best.model]])
+    no.successful.models <- length(which(!is.na(criteria)))
+    if(no.successful.models > 0){
+      best.model.index <- which(criteria == min(na.omit(criteria)))
+      best.model <- results[[best.model.index]]
+      best.model$model.index <- best.model.index
+      #If there is more than one sucessful model find the delta criteria to the 
+      #second best model.
+      if(no.successful.models > 1){
+        sorted.criteria <- sort(criteria, na.last = NA)
+        delta.criteria <- sorted.criteria[2] - sorted.criteria[1]
+        best.model$delta.criteria <- delta.criteria
+      }
+      return(best.model)
     }else{
       return(NULL)
     }
