@@ -4,12 +4,12 @@ is.gap <- function(poly, poly.list){
     true.vals <- which(pts.check)
     if(length(true.vals) == length(pts.check)){
       return(TRUE)
-    }else if(length(true.vals) == length(pts.check)-1){
-      warning("Two of the polygons have a shared boundary point.", call. = FALSE, immediate. = TRUE)
-      return(TRUE)
+      #}else if(length(true.vals) == length(pts.check)-1){
+      #  warning("Two of the polygons have a shared boundary point.", call. = FALSE, immediate. = TRUE)
+      #  return(TRUE)
     }else if(length(true.vals) > 0){
-      print(poly)
-      stop(paste("Some of the polygons in the shapefile are intersecting.",sep = ""), call. = FALSE)    
+      #print(cbind(poly,pts.check))
+      stop(paste("Some of the polygons in the shapefile are intersecting.",sep = ""), call. = FALSE)
     }else{
       return(FALSE)
     }
@@ -17,7 +17,7 @@ is.gap <- function(poly, poly.list){
   pts <- as.points(poly)
   pts.in <- array(NA, dim = c(nrow(pts),length(poly.list)))
   for(p in seq(along = poly.list)){
-    pts.in[,p] <- inout(as.points(pts), poly.list[[p]], bound = FALSE)
+    pts.in[,p] <- inout(as.points(pts), poly.list[[p]], bound = TRUE)
   }
   poly.inside <- apply(pts.in, 2, FUN = all.true)
   if(length(which(poly.inside)) == 1){
