@@ -1,6 +1,6 @@
-uf.detect <- function(perp.dists, detectability){
+uf.detect <- function(dists, detectability){
   #for now just implement a straightforward hn detection function
-  x <- perp.dists
+  x <- dists
   if(length(detectability@scale.param) > 1){
     probs <- sample(detectability@scale.param, length(x), replace = TRUE)
   }else{
